declare class Node {
    #private;
    static traversal: {
        readonly depth: "depth";
        readonly breadth: "breadth";
    };
    /**
     * The parent of the node.
     *
     * If the node is a root node, this will be `null`.
     */
    get parent(): this | null;
    /**
     * The depth of the node in the tree.
     *
     * The depth of a root node is `0`.
     */
    get depth(): number;
    /**
     * The size of the node.
     *
     * The size of a node is the number of nodes in the subtree rooted at the node.
     */
    get size(): number;
    /**
     * An iterator that yields the ancestors of the node.
     *
     * The first ancestor is the parent of the node.
     *
     * @returns An iterator that yields the ancestors of the node.
     */
    ancestors(): IterableIterator<this>;
    /**
     * An iterator that yields the children of the node.
     *
     * @returns An iterator that yields the children of the node.
     */
    children(): IterableIterator<this>;
    /**
     * An iterator that yields the descendants of the node.
     *
     * @param traverse - The traversal order of the descendants.
     * @returns An iterator that yields the descendants of the node.
     */
    descendants(traverse?: 'depth' | 'breadth'): IterableIterator<this>;
    /**
     * An iterator that yields the leaves of the node.
     *
     * @returns An iterator that yields the leaves of the node.
     */
    leaves(): IterableIterator<this>;
    /**
     * The next sibling of the node.
     *
     * If the node is the last child of the parent (or does not have a parent), this will be `null`.
     */
    get nextSibling(): Node | null;
    /**
     * The previous sibling of the node.
     *
     * If the node is the first child of the parent (or does not have a parent), this will be `null`.
     */
    get previousSibling(): Node | null;
    /**
     * The first child of the node.
     *
     * If the node does not have any children, this will be `null`.
     */
    get firstChild(): Node | null;
    /**
     * The last child of the node.
     *
     * If the node does not have any children, this will be `null`.
     */
    get lastChild(): Node | null;
    /**
     * Appends a child to the node.
     *
     * @param child - The child to append.
     * @returns The appended child.
     */
    appendChild(child: this): this;
    /**
     * Inserts a child before a node.
     *
     * If the node is `null`, the child will be appended to the node.
     *
     * If the child is already a child of the node, it will be moved to the new position.
     *
     * If the child is a child of another node, it will be removed from that node.
     *
     * If the node is not a child of this node, an error will be thrown.
     *
     * @param child - The child to append.
     * @param before - The node to insert the child before.
     * @returns The appended child.
     */
    insertBefore(child: this, before: this | null): this;
    /**
     * Removes a child from the node.
     *
     * If the child is not a child of the node, an error will be thrown.
     *
     * @param child - The child to remove.
     * @returns The removed child.
     */
    removeChild(child: this): this;
    /**
     * Detaches the node from the tree.
     *
     * If the node is a root node, this will have no effect.
     *
     * This is the same as calling `removeChild` on the parent of the node.
     *
     * @see Node#removeChild
     */
    detach(): void;
    /**
     * Checks if the node is an ancestor of this node.
     *
     * @param node - The node to check if it is an ancestor of this node.
     * @returns Whether or not the node is an ancestor of this node.
     */
    isAncestorOf(node: Node): boolean;
    /**
     * Checks if the node is a descendant of this node.
     *
     * @param node - The node to check if it is a descendant of this node.
     * @returns Whether or not the node is a descendant of this node.
     */
    isDescendantOf(node: Node): boolean;
    /**
     * Checks if the node is a sibling of this node.
     *
     * @param node - The node to check if it is a sibling of this node.
     * @returns Whether or not the node is a sibling of this node.
     */
    isSiblingOf(node: Node): boolean;
    /**
     * Checks if the node is a leaf node.
     */
    get isLeaf(): boolean;
    /**
     * Checks if the node is a root node.
     */
    get isRoot(): boolean;
    /**
     * Checks if the node is the first child of its parent.
     *
     * If the node is a root node, this will be `false`.
     */
    get isFirstChild(): boolean;
    /**
     * Checks if the node is the last child of its parent.
     *
     * If the node is a root node, this will be `false`.
     */
    get isLastChild(): boolean;
    /**
     * Finds the common ancestor of the node and another node.
     *
     * @param node - The other node.
     * @returns The common ancestor of the node and the other node, or `null` if there is no common ancestor.
     */
    static commonAncestor(a: Node, b: Node): Node | null;
}

export { Node };
